@@ -160,10 +160,10 @@ void vtkXMLDataElement::SetName(const char* _arg)
     this->Name = cp1;
     do { *cp1++ = *cp2++; } while ( --n );
   }
-   else
-   {
+  else
+  {
     this->Name = nullptr;
-   }
+  }
   this->Modified();
 }
 
@@ -397,7 +397,7 @@ void vtkXMLDataElement::PrintCharacterData(ostream &os, vtkIndent indent)
 // &lt;, &gt;, &amp;, &quot;, &apos;, respectively.
 void vtkXMLDataElement::PrintWithEscapedData(ostream& os, const char* data)
 {
-  for(size_t i=0;i<strlen(data);i++)
+  for(size_t i=0; data[i] != 0; i++)
   {
     switch(data[i])
     {
@@ -971,12 +971,17 @@ int vtkXMLDataElement::GetWordTypeAttribute(const char* name, int& value)
     value = VTK_STRING;
     return 1;
   }
+  else if (strcmp(v, "Bit") == 0)
+  {
+    value = VTK_BIT;
+    return 1;
+  }
   else
   {
     vtkErrorMacro("Unknown data type \"" << v << "\".  Supported types are:\n"
                   "Int8,  Int16,  Int32,  Int64,\n"
                   "UInt8, UInt16, UInt32, UInt64,\n"
-                  "Float32, Float64, String\n");
+                  "Float32, Float64, String, Bit\n");
     return 0;
   }
 }
