@@ -19,6 +19,7 @@
 #include "vtkBoundingBox.h"
 #include "vtkCellData.h"
 #include "vtkColorTransferFunction.h"
+#include "vtkCommand.h"
 #include "vtkCompositeDataDisplayAttributes.h"
 #include "vtkCompositeDataIterator.h"
 #include "vtkCompositeDataPipeline.h"
@@ -45,6 +46,7 @@
 #include "vtkShaderProgram.h"
 #include "vtkTextureObject.h"
 #include "vtkTransform.h"
+#include "vtkUnsignedIntArray.h"
 
 #include <algorithm>
 #include <sstream>
@@ -122,32 +124,27 @@ void vtkCompositeMapperHelper2::SetShaderValues(
 
   if (useNanColor)
   {
-    float nanAmbient[3] = {
-      static_cast<float>(nanColor[0] * this->CurrentAmbientIntensity),
-      static_cast<float>(nanColor[1] * this->CurrentAmbientIntensity),
-      static_cast<float>(nanColor[2] * this->CurrentAmbientIntensity)
+    float fnancolor[3] = {
+      static_cast<float>(nanColor[0]),
+      static_cast<float>(nanColor[1]),
+      static_cast<float>(nanColor[2])
     };
-    float nanDiffuse[3] = {
-      static_cast<float>(nanColor[0] * this->CurrentDiffuseIntensity),
-      static_cast<float>(nanColor[1] * this->CurrentDiffuseIntensity),
-      static_cast<float>(nanColor[2] * this->CurrentDiffuseIntensity)
-    };
-    prog->SetUniform3f("ambientColorUniform", nanAmbient);
-    prog->SetUniform3f("diffuseColorUniform", nanDiffuse);
+    prog->SetUniform3f("ambientColorUniform", fnancolor);
+    prog->SetUniform3f("diffuseColorUniform", fnancolor);
   }
   else
   {
     vtkColor3d &aColor = hdata->AmbientColor;
     float ambientColor[3] = {
-      static_cast<float>(aColor[0] * this->CurrentAmbientIntensity),
-      static_cast<float>(aColor[1] * this->CurrentAmbientIntensity),
-      static_cast<float>(aColor[2] * this->CurrentAmbientIntensity)
+      static_cast<float>(aColor[0]),
+      static_cast<float>(aColor[1]),
+      static_cast<float>(aColor[2])
     };
     vtkColor3d &dColor = hdata->DiffuseColor;
     float diffuseColor[3] = {
-      static_cast<float>(dColor[0] * this->CurrentDiffuseIntensity),
-      static_cast<float>(dColor[1] * this->CurrentDiffuseIntensity),
-      static_cast<float>(dColor[2] * this->CurrentDiffuseIntensity)
+      static_cast<float>(dColor[0]),
+      static_cast<float>(dColor[1]),
+      static_cast<float>(dColor[2])
     };
     prog->SetUniform3f("ambientColorUniform", ambientColor);
     prog->SetUniform3f("diffuseColorUniform", diffuseColor);
@@ -158,6 +155,17 @@ void vtkCompositeMapperHelper2::SetShaderValues(
   }
 }
 
+void vtkCompositeMapperHelper2::UpdateShaders(
+  vtkOpenGLHelper &cellBO, vtkRenderer *ren, vtkActor *act)
+{
+  Superclass::UpdateShaders(cellBO, ren, act);
+  if (cellBO.Program && this->Parent)
+  {
+    // allow the program to set what it wants
+    this->Parent->InvokeEvent(vtkCommand::UpdateShaderEvent, cellBO.Program);
+  }
+}
+
 void vtkCompositeMapperHelper2::ReplaceShaderColor(
   std::map<vtkShader::Type, vtkShader *> shaders,
   vtkRenderer *ren, vtkActor *actor)
@@ -173,8 +181,8 @@ void vtkCompositeMapperHelper2::ReplaceShaderColor(
     vtkShaderProgram::Substitute(FSSource,"//VTK::Color::Impl",
       "//VTK::Color::Impl\n"
       "  if (OverridesColor) {\n"
-      "    ambientColor = ambientColorUniform;\n"
-      "    diffuseColor = diffuseColorUniform; }\n",
+      "    ambientColor = ambientColorUniform * ambientIntensity;\n"
+      "    diffuseColor = diffuseColorUniform * diffuseIntensity; }\n",
       false);
 
     shaders[vtkShader::Fragment]->SetSource(FSSource);
@@ -298,9 +306,6 @@ void vtkCompositeMapperHelper2::DrawIBO(
 {
   if (CellBO.IBO->IndexCount)
   {
-    bool haveCellTexture =
-      (this->HaveCellScalars || this->HaveCellNormals || this->HavePickScalars);
-
     if (pointSize > 0)
     {
 #if GL_ES_VERSION_3_0 != 1
@@ -338,23 +343,12 @@ void vtkCompositeMapperHelper2::DrawIBO(
 
     this->RenderedList.clear();
     bool selecting = (this->CurrentSelector ? true : false);
-    for (dataIter it = this->Data.begin(); it != this->Data.end(); )
+    for (dataIter it = this->Data.begin(); it != this->Data.end(); ++it)
     {
       vtkCompositeMapperHelperData *starthdata = it->second;
-      vtkCompositeMapperHelperData *endhdata = starthdata;
-      do
-      {
-        this->RenderedList.push_back(it->first);
-        endhdata = it->second;
-        ++it;
-      }
-      while (it != this->Data.end() &&
-        !(this->HaveAppleBug && haveCellTexture) &&
-        !endhdata->Different(it->second, this->CurrentSelector,
-          haveCellTexture ? primType : -1));
-      if (endhdata->Visibility &&
-          ((selecting && endhdata->Pickability) || !selecting) &&
-          endhdata->NextIndex[primType] > starthdata->StartIndex[primType])
+      if (starthdata->Visibility &&
+          ((selecting && starthdata->Pickability) || !selecting) &&
+          starthdata->NextIndex[primType] > starthdata->StartIndex[primType])
       {
         //compilers think this can exceed the bounds so we also
         // test against primType even though we should not need to
@@ -365,8 +359,8 @@ void vtkCompositeMapperHelper2::DrawIBO(
         }
         glDrawRangeElements(mode,
           static_cast<GLuint>(starthdata->StartVertex),
-          static_cast<GLuint>(endhdata->NextVertex > 0 ? endhdata->NextVertex - 1 : 0),
-          static_cast<GLsizei>(endhdata->NextIndex[primType] - starthdata->StartIndex[primType]),
+          static_cast<GLuint>(starthdata->NextVertex > 0 ? starthdata->NextVertex - 1 : 0),
+          static_cast<GLsizei>(starthdata->NextIndex[primType] - starthdata->StartIndex[primType]),
           GL_UNSIGNED_INT,
           reinterpret_cast<const GLvoid *>(starthdata->StartIndex[primType]*sizeof(GLuint)));
       }
@@ -392,10 +386,6 @@ void vtkCompositeMapperHelper2::RenderPieceDraw(
     pointPicking = true;
   }
 
-  vtkProperty *ppty = actor->GetProperty();
-  this->CurrentAmbientIntensity = ppty->GetAmbient();
-  this->CurrentDiffuseIntensity = ppty->GetDiffuse();
-
   this->PrimitiveIDOffset = 0;
 
   // draw IBOs
@@ -410,13 +400,11 @@ void vtkCompositeMapperHelper2::RenderPieceDraw(
   }
 
   if (this->CurrentSelector && (
-        this->CurrentSelector->GetCurrentPass() == vtkHardwareSelector::ID_LOW24 ||
-        this->CurrentSelector->GetCurrentPass() == vtkHardwareSelector::ID_MID24 ||
-        this->CurrentSelector->GetCurrentPass() == vtkHardwareSelector::ID_HIGH16))
+        this->CurrentSelector->GetCurrentPass() == vtkHardwareSelector::CELL_ID_LOW24 ||
+        this->CurrentSelector->GetCurrentPass() == vtkHardwareSelector::CELL_ID_HIGH24))
   {
-    this->CurrentSelector->RenderAttributeId(this->PrimitiveIDOffset);
+    this->CurrentSelector->SetPropColorValue(this->PrimitiveIDOffset);
   }
-
 }
 
 vtkCompositeMapperHelperData *vtkCompositeMapperHelper2::AddData(
@@ -438,43 +426,6 @@ vtkCompositeMapperHelperData *vtkCompositeMapperHelper2::AddData(
   return found->second;
 }
 
-//-------------------------------------------------------------------------
-bool vtkCompositeMapperHelper2::GetNeedToRebuildBufferObjects(
-  vtkRenderer * /* ren */, vtkActor *act)
-{
-  // might need to loop of Data and check each one?  Or is that handled
-  // in parent Render
-
-
-  // we use a string instead of just mtime because
-  // we do not want to check the actor's mtime.  Actor
-  // changes mtime every time it's position changes. But
-  // changing an actor's position does not require us to
-  // rebuild all the VBO/IBOs. So we only watch the mtime
-  // of the property/texture. But if someone changes the
-  // Property on an actor the mtime may actually go down
-  // because the new property has an older mtime. So
-  // we watch the actual mtime, to see if it changes as
-  // opposed to just checking if it is greater.
-  std::ostringstream toString;
-  toString.str("");
-  toString.clear();
-  toString <<
-    act->GetProperty()->GetMTime() <<
-    'A' << (this->CurrentInput ? this->CurrentInput->GetMTime() : 0) <<
-    'B' << (act->GetTexture() ? act->GetTexture()->GetMTime() : 0);
-
-  if (this->VBOBuildString != toString.str() ||
-      this->VBOBuildTime < this->SelectionStateChanged ||
-      this->VBOBuildTime < this->GetMTime() ||
-      (this->CurrentInput && this->VBOBuildTime < this->CurrentInput->GetMTime()))
-  {
-    this->VBOBuildString = toString.str();
-    return true;
-  }
-  return false;
-}
-
 //-------------------------------------------------------------------------
 void vtkCompositeMapperHelper2::BuildBufferObjects(
   vtkRenderer *ren,
@@ -500,7 +451,6 @@ void vtkCompositeMapperHelper2::BuildBufferObjects(
   this->AppleBugPrimIDs.resize(0);
 
   dataIter iter;
-  unsigned int voffset = 0;
   this->VBOs->ClearAllVBOs();
 
   if (this->Data.begin() == this->Data.end())
@@ -520,15 +470,17 @@ void vtkCompositeMapperHelper2::BuildBufferObjects(
     hdata->Data->GetPoints()->GetBounds(bounds);
     bbox.AddBounds(bounds);
 
-    hdata->StartVertex = voffset;
     for (int i = 0; i < PrimitiveEnd; i++)
     {
       hdata->StartIndex[i] =
         static_cast<unsigned int>(this->IndexArray[i].size());
     }
+
+    vtkIdType voffset = 0;
     this->AppendOneBufferObject(ren, act, hdata,
       voffset, newColors, newNorms);
-    hdata->NextVertex = voffset;
+    hdata->StartVertex = static_cast<unsigned int>(voffset);
+    hdata->NextVertex = hdata->StartVertex + hdata->Data->GetPoints()->GetNumberOfPoints();
     for (int i = 0; i < PrimitiveEnd; i++)
     {
       hdata->NextIndex[i] =
@@ -536,6 +488,13 @@ void vtkCompositeMapperHelper2::BuildBufferObjects(
     }
   }
 
+  // clear color cache
+  for (auto& c : this->ColorArrayMap)
+  {
+    c.second->Delete();
+  }
+  this->ColorArrayMap.clear();
+
   vtkOpenGLVertexBufferObject *posVBO = this->VBOs->GetVBO("vertexMC");
   if (posVBO && this->ShiftScaleMethod ==
       vtkOpenGLVertexBufferObject::AUTO_SHIFT_SCALE)
@@ -576,7 +535,7 @@ void vtkCompositeMapperHelper2::BuildBufferObjects(
   }
 
   // allocate as needed
-  if (this->HaveCellScalars || this->HavePickScalars)
+  if (this->HaveCellScalars)
   {
     if (!this->CellScalarTexture)
     {
@@ -637,16 +596,8 @@ void vtkCompositeMapperHelper2::BuildBufferObjects(
     }
   }
 
-  vtkHardwareSelector* selector = ren->GetSelector();
-  bool pointPicking = false;
-  if (selector && this->PopulateSelectionSettings &&
-      selector->GetFieldAssociation() == vtkDataObject::FIELD_ASSOCIATION_POINTS)
-  {
-    pointPicking = true;
-  }
   if (this->HaveAppleBug &&
-      !pointPicking &&
-      (this->HaveCellNormals || this->HaveCellScalars || this->HavePickScalars))
+      (this->HaveCellNormals || this->HaveCellScalars))
   {
     if (!this->AppleBugPrimIDBuffer)
     {
@@ -666,7 +617,7 @@ void vtkCompositeMapperHelper2::AppendOneBufferObject(
   vtkRenderer *ren,
   vtkActor *act,
   vtkCompositeMapperHelperData *hdata,
-  unsigned int &voffset,
+  vtkIdType &voffset,
   std::vector<unsigned char> &newColors,
   std::vector<float> &newNorms
   )
@@ -741,12 +692,10 @@ void vtkCompositeMapperHelper2::AppendOneBufferObject(
   int representation = act->GetProperty()->GetRepresentation();
   vtkHardwareSelector* selector = ren->GetSelector();
 
-  bool pointPicking = false;
   if (selector && this->PopulateSelectionSettings &&
       selector->GetFieldAssociation() == vtkDataObject::FIELD_ASSOCIATION_POINTS)
   {
     representation = VTK_POINTS;
-    pointPicking = true;
   }
 
   // if we have cell scalars then we have to
@@ -762,13 +711,14 @@ void vtkCompositeMapperHelper2::AppendOneBufferObject(
   hdata->PrimOffsets[1] = hdata->PrimOffsets[0] +
     prims[0]->GetNumberOfConnectivityEntries() -
     prims[0]->GetNumberOfCells();
-  hdata->PrimOffsets[2] = hdata->PrimOffsets[1] +
-    prims[1]->GetNumberOfConnectivityEntries() -
-    2*prims[1]->GetNumberOfCells();
 
   this->AppendCellTextures(ren, act, prims, representation,
     newColors, newNorms, poly);
 
+  hdata->PrimOffsets[2] = hdata->PrimOffsets[1] +
+    prims[1]->GetNumberOfConnectivityEntries() -
+    2*prims[1]->GetNumberOfCells();
+
   hdata->PrimOffsets[4] = (!newColors.empty() ? newColors.size()/4 : newNorms.size()/4);
 
   // we back compute the strip number
@@ -776,11 +726,10 @@ void vtkCompositeMapperHelper2::AppendOneBufferObject(
     - 3*prims[3]->GetNumberOfCells();
   hdata->PrimOffsets[3] = hdata->PrimOffsets[4] - triCount;
 
-  // on apple with the AMD PrimID bug we use a slow
-  // painful approach to work around it
+  // on Apple Macs with the AMD PrimID bug <rdar://20747550>
+  // we use a slow painful approach to work around it (pre 10.11).
   if (this->HaveAppleBug &&
-      !pointPicking &&
-      (this->HaveCellNormals || this->HaveCellScalars || this->HavePickScalars))
+      (this->HaveCellNormals || this->HaveCellScalars))
   {
     poly = this->HandleAppleBug(poly, this->AppleBugPrimIDs);
     prims[0] =  poly->GetVerts();
@@ -833,12 +782,58 @@ void vtkCompositeMapperHelper2::AppendOneBufferObject(
     }
   }
 
+  // Check if color array is already computed for the current array.
+  // This step is mandatory otherwise the test ArrayExists will fail for "scalarColor" even if
+  // the array used to map the color has already been added.
+  if (c)
+  {
+    int cellFlag = 0; // not used
+    vtkAbstractArray* abstractArray = this->GetAbstractScalars(poly, this->ScalarMode,
+      this->ArrayAccessMode, this->ArrayId, this->ArrayName, cellFlag);
+
+    auto iter = this->ColorArrayMap.find(abstractArray);
+    if (iter != this->ColorArrayMap.end())
+    {
+      c = iter->second;
+    }
+    else
+    {
+      this->ColorArrayMap[abstractArray] = c;
+      c->Register(this);
+    }
+  }
+
   // Build the VBO
-  this->VBOs->AppendDataArray(
-    "vertexMC", poly->GetPoints()->GetData(), VTK_FLOAT);
-  this->VBOs->AppendDataArray("normalMC", n, VTK_FLOAT);
-  this->VBOs->AppendDataArray("scalarColor", c, VTK_UNSIGNED_CHAR);
-  this->VBOs->AppendDataArray("tcoordMC", tcoords, VTK_FLOAT);
+  vtkIdType offsetPos = 0;
+  vtkIdType offsetNorm = 0;
+  vtkIdType offsetColor = 0;
+  vtkIdType offsetTex = 0;
+  vtkIdType totalOffset = 0;
+  vtkIdType dummy = 0;
+  bool exists =
+    this->VBOs->ArrayExists("vertexMC", poly->GetPoints()->GetData(), offsetPos, totalOffset) &&
+    this->VBOs->ArrayExists("normalMC", n, offsetNorm, dummy) &&
+    this->VBOs->ArrayExists("scalarColor", c, offsetColor,dummy) &&
+    this->VBOs->ArrayExists("tcoord", tcoords, offsetTex, dummy);
+
+  // if all used arrays have the same offset and have already been added,
+  // we can reuse them and save memory
+  if (exists &&
+    (offsetNorm == 0 || offsetPos == offsetNorm) &&
+    (offsetColor == 0 || offsetPos == offsetColor) &&
+    (offsetTex == 0 || offsetPos == offsetTex))
+  {
+    voffset = offsetPos;
+  }
+  else
+  {
+    this->VBOs->AppendDataArray("vertexMC", poly->GetPoints()->GetData(), VTK_FLOAT);
+    this->VBOs->AppendDataArray("normalMC", n, VTK_FLOAT);
+    this->VBOs->AppendDataArray("scalarColor", c, VTK_UNSIGNED_CHAR);
+    this->VBOs->AppendDataArray("tcoord", tcoords, VTK_FLOAT);
+
+    voffset = totalOffset;
+  }
 
   // now create the IBOs
   vtkOpenGLIndexBufferObject::AppendPointIndexBuffer(
@@ -874,7 +869,7 @@ void vtkCompositeMapperHelper2::AppendOneBufferObject(
         }
         if (ef && !ef->IsA("vtkUnsignedCharArray"))
         {
-          vtkDebugMacro(<< "Currently only unsigned char edge flags are suported.");
+          vtkDebugMacro(<< "Currently only unsigned char edge flags are supported.");
           ef = nullptr;
         }
       }
@@ -891,8 +886,8 @@ void vtkCompositeMapperHelper2::AppendOneBufferObject(
       vtkOpenGLIndexBufferObject::AppendStripIndexBuffer(
         this->IndexArray[3], prims[3], voffset, true);
     }
-   else // SURFACE
-   {
+    else // SURFACE
+    {
       vtkOpenGLIndexBufferObject::AppendTriangleIndexBuffer(
         this->IndexArray[2],
         prims[2],
@@ -900,7 +895,7 @@ void vtkCompositeMapperHelper2::AppendOneBufferObject(
         voffset);
       vtkOpenGLIndexBufferObject::AppendStripIndexBuffer(
         this->IndexArray[3], prims[3], voffset, false);
-   }
+    }
   }
 
   // when drawing edges also build the edge IBOs
@@ -918,7 +913,7 @@ void vtkCompositeMapperHelper2::AppendOneBufferObject(
       }
       if (!ef->IsA("vtkUnsignedCharArray"))
       {
-        vtkDebugMacro(<< "Currently only unsigned char edge flags are suported.");
+        vtkDebugMacro(<< "Currently only unsigned char edge flags are supported.");
         ef = nullptr;
       }
     }
@@ -942,9 +937,6 @@ void vtkCompositeMapperHelper2::AppendOneBufferObject(
       this->IndexArray[PrimitiveVertices], prims, voffset);
   }
 
-
-  voffset += poly->GetPoints()->GetNumberOfPoints();
-
   // free up polydata if allocated due to apple bug
   if (poly != hdata->Data)
   {
@@ -952,6 +944,389 @@ void vtkCompositeMapperHelper2::AppendOneBufferObject(
   }
 }
 
+void vtkCompositeMapperHelper2::ProcessSelectorPixelBuffers(
+  vtkHardwareSelector *sel,
+  std::vector<unsigned int> &pixeloffsets,
+  vtkProp *prop)
+{
+  if (!this->PopulateSelectionSettings)
+  {
+    return;
+  }
+
+  if (sel->GetCurrentPass() == vtkHardwareSelector::ACTOR_PASS)
+  {
+    this->PickPixels.clear();
+    return;
+  }
+
+  if (PickPixels.size() == 0 && pixeloffsets.size())
+  {
+    // preprocess the image to find matching pixels and
+    // store them in a map of vectors based on flat index
+    // this makes the block processing far faster as we just
+    // loop over the pixels for our block
+    unsigned char *compositedata =
+      sel->GetRawPixelBuffer(vtkHardwareSelector::COMPOSITE_INDEX_PASS);
+
+    if (!compositedata)
+    {
+      return;
+    }
+
+    size_t maxFlatIndex = 0;
+    for (dataIter it = this->Data.begin(); it != this->Data.end(); ++it)
+    {
+      maxFlatIndex = (it->second->FlatIndex > maxFlatIndex) ? it->second->FlatIndex : maxFlatIndex;
+    }
+
+    this->PickPixels.resize(maxFlatIndex + 1);
+
+    for (auto pos : pixeloffsets)
+    {
+      unsigned int compval = compositedata[pos+2];
+      compval = compval << 8;
+      compval |= compositedata[pos+1];
+      compval = compval << 8;
+      compval |= compositedata[pos];
+      compval -= 1;
+      if (compval <= maxFlatIndex)
+      {
+        this->PickPixels[compval].push_back(pos);
+      }
+    }
+  }
+
+  // for each block update the image
+  for (dataIter it = this->Data.begin(); it != this->Data.end(); ++it)
+  {
+    if (this->PickPixels[it->second->FlatIndex].size())
+    {
+      this->ProcessCompositePixelBuffers(sel, prop, it->second,
+        this->PickPixels[it->second->FlatIndex]);
+    }
+  }
+}
+
+namespace
+{
+  unsigned int convertToCells(
+    unsigned int *offset,
+    unsigned int *stride,
+    unsigned int inval )
+  {
+    if (inval < offset[0])
+    {
+      return inval;
+    }
+    if (inval < offset[1])
+    {
+      return offset[0] + (inval - offset[0]) / stride[0];
+    }
+    return offset[0] + (offset[1] - offset[0]) / stride[0] + (inval - offset[1]) / stride[1];
+  }
+}
+
+void vtkCompositeMapperHelper2::ProcessCompositePixelBuffers(
+  vtkHardwareSelector *sel,
+  vtkProp *prop,
+  vtkCompositeMapperHelperData *hdata,
+  std::vector<unsigned int> &pixeloffsets)
+{
+  vtkPolyData *poly = hdata->Data;
+
+  if (!poly)
+  {
+    return;
+  }
+
+  // which pass are we processing ?
+  int currPass = sel->GetCurrentPass();
+
+  // get some common useful values
+  bool pointPicking =
+    sel->GetFieldAssociation() == vtkDataObject::FIELD_ASSOCIATION_POINTS;
+  vtkPointData *pd = poly->GetPointData();
+  vtkCellData *cd = poly->GetCellData();
+
+  // get some values
+  unsigned char *rawplowdata = sel->GetRawPixelBuffer(vtkHardwareSelector::POINT_ID_LOW24);
+  unsigned char *rawphighdata = sel->GetRawPixelBuffer(vtkHardwareSelector::POINT_ID_HIGH24);
+
+  // do we need to do anything to the process pass data?
+  if (currPass == vtkHardwareSelector::PROCESS_PASS)
+  {
+    unsigned char *processdata = sel->GetPixelBuffer(vtkHardwareSelector::PROCESS_PASS);
+    vtkUnsignedIntArray* processArray = nullptr;
+
+    if (sel->GetUseProcessIdFromData())
+    {
+      processArray = this->ProcessIdArrayName ?
+        vtkArrayDownCast<vtkUnsignedIntArray>(
+          pd->GetArray(this->ProcessIdArrayName)) : nullptr;
+    }
+
+    if (processArray && processdata && rawplowdata)
+    {
+      for (auto pos : pixeloffsets)
+      {
+        unsigned int inval = 0;
+        if (rawphighdata)
+        {
+          inval = rawphighdata[pos];
+          inval = inval << 8;
+        }
+        inval |= rawplowdata[pos + 2];
+        inval = inval << 8;
+        inval |= rawplowdata[pos + 1];
+        inval = inval << 8;
+        inval |= rawplowdata[pos];
+        inval -= 1;
+        inval -= hdata->StartVertex;
+        unsigned int outval =  processArray->GetValue(inval) + 1;
+        processdata[pos] = outval & 0xff;
+        processdata[pos + 1] = (outval & 0xff00) >> 8;
+        processdata[pos + 2] = (outval & 0xff0000) >> 16;
+      }
+    }
+  }
+
+  // do we need to do anything to the point id data?
+  if (currPass == vtkHardwareSelector::POINT_ID_LOW24)
+  {
+    vtkIdTypeArray* pointArrayId = this->PointIdArrayName ?
+      vtkArrayDownCast<vtkIdTypeArray>(
+        pd->GetArray(this->PointIdArrayName)) : nullptr;
+
+    // do we need to do anything to the point id data?
+    if (rawplowdata)
+    {
+      unsigned char *plowdata = sel->GetPixelBuffer(vtkHardwareSelector::POINT_ID_LOW24);
+
+      for (auto pos : pixeloffsets)
+      {
+        unsigned int inval = 0;
+        if (rawphighdata)
+        {
+          inval = rawphighdata[pos];
+          inval = inval << 8;
+        }
+        inval |= rawplowdata[pos + 2];
+        inval = inval << 8;
+        inval |= rawplowdata[pos + 1];
+        inval = inval << 8;
+        inval |= rawplowdata[pos];
+        inval -= 1;
+        inval -= hdata->StartVertex;
+        vtkIdType outval =  inval + 1;
+        if (pointArrayId)
+        {
+          outval = pointArrayId->GetValue(inval) + 1;
+        }
+        plowdata[pos] = outval & 0xff;
+        plowdata[pos + 1] = (outval & 0xff00) >> 8;
+        plowdata[pos + 2] = (outval & 0xff0000) >> 16;
+      }
+    }
+  }
+
+  if (currPass == vtkHardwareSelector::POINT_ID_HIGH24)
+  {
+    vtkIdTypeArray *pointArrayId = this->PointIdArrayName ?
+      vtkArrayDownCast<vtkIdTypeArray>(
+        pd->GetArray(this->PointIdArrayName)) : nullptr;
+
+    // do we need to do anything to the point id data?
+    if (rawphighdata)
+    {
+      unsigned char *phighdata = sel->GetPixelBuffer(vtkHardwareSelector::POINT_ID_HIGH24);
+
+      for (auto pos : pixeloffsets)
+      {
+        unsigned int inval = 0;
+        inval = rawphighdata[pos];
+        inval = inval << 8;
+        inval |= rawplowdata[pos + 2];
+        inval = inval << 8;
+        inval |= rawplowdata[pos + 1];
+        inval = inval << 8;
+        inval |= rawplowdata[pos];
+        inval -= 1;
+        inval -= hdata->StartVertex;
+        vtkIdType outval =  inval + 1;
+        if (pointArrayId)
+        {
+          outval = pointArrayId->GetValue(inval) + 1;
+        }
+        phighdata[pos] = (outval & 0xff000000) >> 24;
+        phighdata[pos + 1] = (outval & 0xff00000000) >> 32;
+        phighdata[pos + 2] = (outval & 0xff0000000000) >> 40;
+      }
+    }
+  }
+
+  // vars for cell based indexing
+  vtkCellArray *prims[4];
+  prims[0] =  poly->GetVerts();
+  prims[1] =  poly->GetLines();
+  prims[2] =  poly->GetPolys();
+  prims[3] =  poly->GetStrips();
+
+  int representation =
+    static_cast<vtkActor *>(prop)->GetProperty()->GetRepresentation();
+
+  unsigned char *rawclowdata = sel->GetRawPixelBuffer(vtkHardwareSelector::CELL_ID_LOW24);
+  unsigned char *rawchighdata = sel->GetRawPixelBuffer(vtkHardwareSelector::CELL_ID_HIGH24);
+
+  // build the mapping of point primID to cell primID
+  // aka when we render triangles in point picking mode
+  // how do we map primid to what would normally be primid
+  unsigned int offset[2];
+  unsigned int stride[2];
+  offset[0] = static_cast<unsigned int>(this->Primitives[PrimitiveVertices].IBO->IndexCount);
+  stride[0] = representation == VTK_POINTS ? 1 : 2;
+  offset[1] = offset[0] + static_cast<unsigned int>(
+    this->Primitives[PrimitiveLines].IBO->IndexCount);
+  stride[1] = representation == VTK_POINTS ? 1 : representation == VTK_WIREFRAME ? 2 : 3;
+
+  // do we need to do anything to the composite pass data?
+  if (currPass == vtkHardwareSelector::COMPOSITE_INDEX_PASS)
+  {
+    unsigned char *compositedata =
+      sel->GetPixelBuffer(vtkHardwareSelector::COMPOSITE_INDEX_PASS);
+
+    vtkUnsignedIntArray *compositeArray = this->CompositeIdArrayName ?
+      vtkArrayDownCast<vtkUnsignedIntArray>(
+        cd->GetArray(this->CompositeIdArrayName)) : nullptr;
+
+    if (compositedata && compositeArray && rawclowdata)
+    {
+      this->UpdateCellMaps(this->HaveAppleBug,
+        poly,
+        prims,
+        representation,
+        poly->GetPoints());
+
+      for (auto pos : pixeloffsets)
+      {
+        unsigned int inval = 0;
+        if (rawchighdata)
+        {
+          inval = rawchighdata[pos];
+          inval = inval << 8;
+        }
+        inval |= rawclowdata[pos + 2];
+        inval = inval << 8;
+        inval |= rawclowdata[pos + 1];
+        inval = inval << 8;
+        inval |= rawclowdata[pos];
+        inval -= 1;
+        inval -= static_cast<int>(hdata->PrimOffsets[0]);
+        if (pointPicking)
+        {
+          inval = convertToCells(offset, stride, inval);
+        }
+        vtkIdType vtkCellId = this->CellCellMap[inval];
+        unsigned int outval =  compositeArray->GetValue(vtkCellId) + 1;
+        compositedata[pos] = outval & 0xff;
+        compositedata[pos + 1] = (outval & 0xff00) >> 8;
+        compositedata[pos + 2] = (outval & 0xff0000) >> 16;
+      }
+    }
+  }
+
+  if (currPass == vtkHardwareSelector::CELL_ID_LOW24)
+  {
+    vtkIdTypeArray *cellArrayId = this->CellIdArrayName ?
+      vtkArrayDownCast<vtkIdTypeArray>(
+        cd->GetArray(this->CellIdArrayName)) : nullptr;
+    unsigned char *clowdata = sel->GetPixelBuffer(vtkHardwareSelector::CELL_ID_LOW24);
+
+    if (rawclowdata)
+    {
+      this->UpdateCellMaps(this->HaveAppleBug,
+        poly,
+        prims,
+        representation,
+        poly->GetPoints());
+
+      for (auto pos : pixeloffsets)
+      {
+        unsigned int inval = 0;
+        if (rawchighdata)
+        {
+          inval = rawchighdata[pos];
+          inval = inval << 8;
+        }
+        inval |= rawclowdata[pos + 2];
+        inval = inval << 8;
+        inval |= rawclowdata[pos + 1];
+        inval = inval << 8;
+        inval |= rawclowdata[pos];
+        inval -= 1;
+        inval -= static_cast<int>(hdata->PrimOffsets[0]);
+        if (pointPicking)
+        {
+          inval = convertToCells(offset, stride, inval);
+        }
+        vtkIdType outval = this->CellCellMap[inval];
+        if (cellArrayId)
+        {
+          outval = cellArrayId->GetValue(outval);
+        }
+        outval++;
+        clowdata[pos] = outval & 0xff;
+        clowdata[pos + 1] = (outval & 0xff00) >> 8;
+        clowdata[pos + 2] = (outval & 0xff0000) >> 16;
+      }
+    }
+  }
+
+  if (currPass == vtkHardwareSelector::CELL_ID_HIGH24)
+  {
+    vtkIdTypeArray *cellArrayId = this->CellIdArrayName ?
+      vtkArrayDownCast<vtkIdTypeArray>(
+        cd->GetArray(this->CellIdArrayName)) : nullptr;
+    unsigned char *chighdata = sel->GetPixelBuffer(vtkHardwareSelector::CELL_ID_HIGH24);
+
+    if (rawchighdata)
+    {
+      this->UpdateCellMaps(this->HaveAppleBug,
+        poly,
+        prims,
+        representation,
+        poly->GetPoints());
+
+      for (auto pos : pixeloffsets)
+      {
+        unsigned int inval = 0;
+        inval = rawchighdata[pos];
+        inval = inval << 8;
+        inval |= rawclowdata[pos + 2];
+        inval = inval << 8;
+        inval |= rawclowdata[pos + 1];
+        inval = inval << 8;
+        inval |= rawclowdata[pos];
+        inval -= 1;
+        inval -= static_cast<int>(hdata->PrimOffsets[0]);
+        if (pointPicking)
+        {
+          inval = convertToCells(offset, stride, inval);
+        }
+        vtkIdType outval = this->CellCellMap[inval];
+        if (cellArrayId)
+        {
+          outval = cellArrayId->GetValue(outval);
+        }
+        outval++;
+        chighdata[pos] = (outval & 0xff000000) >> 24;
+        chighdata[pos + 1] = (outval & 0xff00000000) >> 32;
+        chighdata[pos + 2] = (outval & 0xff0000000000) >> 40;
+      }
+    }
+  }
+
+}
 
 //===================================================================
 // Now the main class methods
@@ -1305,10 +1680,19 @@ void vtkCompositePolyDataMapper2::CopyMapperValuesToHelper(vtkCompositeMapperHel
   helper->SetCompositeIdArrayName(this->GetCompositeIdArrayName());
   helper->SetProcessIdArrayName(this->GetProcessIdArrayName());
   helper->SetCellIdArrayName(this->GetCellIdArrayName());
+  helper->ClearAllShaderReplacements();
   helper->SetVertexShaderCode(this->GetVertexShaderCode());
   helper->SetGeometryShaderCode(this->GetGeometryShaderCode());
   helper->SetFragmentShaderCode(this->GetFragmentShaderCode());
   helper->SetStatic(1);
+  for (auto& repl : this->UserShaderReplacements)
+  {
+    const vtkShader::ReplacementSpec& spec = repl.first;
+    const vtkShader::ReplacementValue& values = repl.second;
+
+    helper->AddShaderReplacement(spec.ShaderType, spec.OriginalValue, spec.ReplaceFirst,
+      values.Replacement, values.ReplaceAll);
+  }
 }
 
 //-----------------------------------------------------------------------------
@@ -1338,6 +1722,30 @@ void vtkCompositePolyDataMapper2::Render(
 {
   this->RenderedList.clear();
 
+  // Make sure that we have been properly initialized.
+  if (ren->GetRenderWindow()->CheckAbortStatus())
+  {
+    return;
+  }
+
+  if (this->GetInputAlgorithm() == nullptr)
+  {
+    return;
+  }
+
+  if (!this->Static)
+  {
+    this->InvokeEvent(vtkCommand::StartEvent,nullptr);
+    this->GetInputAlgorithm()->Update();
+    this->InvokeEvent(vtkCommand::EndEvent,nullptr);
+  }
+
+  if (this->GetInputDataObject(0, 0) == nullptr)
+  {
+    vtkErrorMacro(<< "No input!");
+    return;
+  }
+
   // the first step is to gather up the polydata based on their
   // signatures (aka have normals, have scalars etc)
   if (this->HelperMTime < this->GetInputDataObject(0, 0)->GetMTime() ||
@@ -1476,8 +1884,7 @@ void vtkCompositePolyDataMapper2::Render(
   if (this->RenderValuesBuildTime < this->GetMTime() ||
       this->RenderValuesBuildTime < actor->GetProperty()->GetMTime() ||
       this->RenderValuesBuildTime < this->VBOBuildTime ||
-      this->RenderValuesBuildTime < this->HelperMTime ||
-      this->RenderValuesBuildTime < this->SelectionStateChanged)
+      this->RenderValuesBuildTime < this->HelperMTime)
   {
     vtkProperty* prop = actor->GetProperty();
 
@@ -1503,6 +1910,7 @@ void vtkCompositePolyDataMapper2::Render(
     this->RenderValuesBuildTime.Modified();
   }
 
+  this->InitializeHelpersBeforeRendering(ren, actor);
 
   for (helpIter hiter = this->Helpers.begin(); hiter != this->Helpers.end(); ++hiter)
   {
@@ -1613,3 +2021,53 @@ void vtkCompositePolyDataMapper2::BuildRenderValues(
     this->BlockState.Visibility.pop();
   }
 }
+
+//----------------------------------------------------------------------------
+void vtkCompositePolyDataMapper2::SetInputArrayToProcess(int idx, vtkInformation* inInfo)
+{
+  this->Superclass::SetInputArrayToProcess(idx, inInfo);
+
+  // set inputs to helpers
+  for (auto& helper : this->Helpers)
+  {
+    helper.second->SetInputArrayToProcess(idx, inInfo);
+  }
+}
+
+//----------------------------------------------------------------------------
+void vtkCompositePolyDataMapper2::SetInputArrayToProcess(
+  int idx, int port, int connection, int fieldAssociation, int attributeType)
+{
+  this->Superclass::SetInputArrayToProcess(idx, port, connection, fieldAssociation, attributeType);
+
+  // set inputs to helpers
+  for (auto& helper : this->Helpers)
+  {
+    helper.second->SetInputArrayToProcess(idx, port, connection, fieldAssociation, attributeType);
+  }
+}
+
+//-----------------------------------------------------------------------------
+void vtkCompositePolyDataMapper2::SetInputArrayToProcess(
+  int idx, int port, int connection, int fieldAssociation, const char* name)
+{
+  this->Superclass::SetInputArrayToProcess(idx, port, connection, fieldAssociation, name);
+
+  // set inputs to helpers
+  for (auto& helper : this->Helpers)
+  {
+    helper.second->SetInputArrayToProcess(idx, port, connection, fieldAssociation, name);
+  }
+}
+
+void vtkCompositePolyDataMapper2::ProcessSelectorPixelBuffers(
+  vtkHardwareSelector *sel,
+  std::vector<unsigned int> &pixeloffsets,
+  vtkProp *prop)
+{
+  // forward to helper
+  for (auto& helper : this->Helpers)
+  {
+    helper.second->ProcessSelectorPixelBuffers(sel, pixeloffsets, prop);
+  }
+}
