@@ -42,8 +42,8 @@ namespace internal
 
 struct ClearBuffers : public vtkm::worklet::WorkletMapField
 {
-  typedef void ControlSignature(FieldOut<>, FieldOut<>);
-  typedef void ExecutionSignature(_1, _2);
+  using ControlSignature = void(FieldOut<>, FieldOut<>);
+  using ExecutionSignature = void(_1, _2);
 
   VTKM_CONT
   ClearBuffers() {}
@@ -57,37 +57,10 @@ struct ClearBuffers : public vtkm::worklet::WorkletMapField
     color[3] = 0.f;
     // The depth is set to slightly larger than 1.0f, ensuring this color value always fails a
     // depth check
-    depth = 1.001f;
+    depth = VTKM_DEFAULT_CANVAS_DEPTH;
   }
 }; // struct ClearBuffers
 
-struct ClearBuffersExecutor
-{
-  typedef vtkm::rendering::Canvas::ColorBufferType ColorBufferType;
-  typedef vtkm::rendering::Canvas::DepthBufferType DepthBufferType;
-
-  ColorBufferType ColorBuffer;
-  DepthBufferType DepthBuffer;
-
-  VTKM_CONT
-  ClearBuffersExecutor(const ColorBufferType& colorBuffer, const DepthBufferType& depthBuffer)
-    : ColorBuffer(colorBuffer)
-    , DepthBuffer(depthBuffer)
-  {
-  }
-
-  template <typename Device>
-  VTKM_CONT bool operator()(Device) const
-  {
-    VTKM_IS_DEVICE_ADAPTER_TAG(Device);
-
-    ClearBuffers worklet;
-    vtkm::worklet::DispatcherMapField<ClearBuffers, Device> dispatcher(worklet);
-    dispatcher.Invoke(this->ColorBuffer, this->DepthBuffer);
-    return true;
-  }
-}; // struct ClearBuffersExecutor
-
 struct BlendBackground : public vtkm::worklet::WorkletMapField
 {
   vtkm::Vec<vtkm::Float32, 4> BackgroundColor;
@@ -98,8 +71,8 @@ struct BlendBackground : public vtkm::worklet::WorkletMapField
   {
   }
 
-  typedef void ControlSignature(FieldInOut<>);
-  typedef void ExecutionSignature(_1);
+  using ControlSignature = void(FieldInOut<>);
+  using ExecutionSignature = void(_1);
 
   VTKM_EXEC void operator()(vtkm::Vec<vtkm::Float32, 4>& color) const
   {
@@ -114,36 +87,10 @@ struct BlendBackground : public vtkm::worklet::WorkletMapField
   }
 }; // struct BlendBackground
 
-struct BlendBackgroundExecutor
-{
-  typedef vtkm::rendering::Canvas::ColorBufferType ColorBufferType;
-
-  ColorBufferType ColorBuffer;
-  BlendBackground Worklet;
-
-  VTKM_CONT
-  BlendBackgroundExecutor(const ColorBufferType& colorBuffer,
-                          const vtkm::Vec<vtkm::Float32, 4>& backgroundColor)
-    : ColorBuffer(colorBuffer)
-    , Worklet(backgroundColor)
-  {
-  }
-
-  template <typename Device>
-  VTKM_CONT bool operator()(Device) const
-  {
-    VTKM_IS_DEVICE_ADAPTER_TAG(Device);
-
-    vtkm::worklet::DispatcherMapField<BlendBackground, Device> dispatcher(this->Worklet);
-    dispatcher.Invoke(this->ColorBuffer);
-    return true;
-  }
-}; // struct BlendBackgroundExecutor
-
 struct DrawColorSwatch : public vtkm::worklet::WorkletMapField
 {
-  typedef void ControlSignature(FieldIn<>, WholeArrayInOut<>);
-  typedef void ExecutionSignature(_1, _2);
+  using ControlSignature = void(FieldIn<>, WholeArrayInOut<>);
+  using ExecutionSignature = void(_1, _2);
 
   VTKM_CONT
   DrawColorSwatch(vtkm::Id2 dims,
@@ -185,8 +132,8 @@ struct DrawColorSwatch : public vtkm::worklet::WorkletMapField
 
 struct DrawColorBar : public vtkm::worklet::WorkletMapField
 {
-  typedef void ControlSignature(FieldIn<>, WholeArrayInOut<>, WholeArrayIn<>);
-  typedef void ExecutionSignature(_1, _2, _3);
+  using ControlSignature = void(FieldIn<>, WholeArrayInOut<>, WholeArrayIn<>);
+  using ExecutionSignature = void(_1, _2, _3);
 
   VTKM_CONT
   DrawColorBar(vtkm::Id2 dims, vtkm::Id2 xBounds, vtkm::Id2 yBounds, bool horizontal)
@@ -207,10 +154,15 @@ struct DrawColorBar : public vtkm::worklet::WorkletMapField
   {
     // local bar coord
     vtkm::Id x = index % BarWidth;
-    vtkm::Id y = index / BarWidth, yLocal = y;
+    vtkm::Id y = index / BarWidth;
     vtkm::Id sample = Horizontal ? x : y;
-    vtkm::Vec<vtkm::Float32, 4> color = colorMap.Get(sample);
 
+
+    const vtkm::Vec<vtkm::UInt8, 4> color = colorMap.Get(sample);
+
+    vtkm::Float32 normalizedHeight = Horizontal
+      ? static_cast<vtkm::Float32>(y) / static_cast<vtkm::Float32>(BarHeight)
+      : static_cast<vtkm::Float32>(x) / static_cast<vtkm::Float32>(BarWidth);
     // offset to global image coord
     x += BarBottomLeft[0];
     y += BarBottomLeft[1];
@@ -218,23 +170,29 @@ struct DrawColorBar : public vtkm::worklet::WorkletMapField
     vtkm::Id offset = y * ImageWidth + x;
     // If the colortable has alpha values, we blend each color sample with translucent white.
     // The height of the resultant translucent bar indicates the opacity.
-    vtkm::Float32 normalizedHeight = static_cast<vtkm::Float32>(yLocal) / BarHeight;
-    if (color[3] < 1.0f && normalizedHeight <= color[3])
+
+    constexpr vtkm::Float32 conversionToFloatSpace = (1.0f / 255.0f);
+    vtkm::Float32 alpha = color[3] * conversionToFloatSpace;
+    if (alpha < 1 && normalizedHeight <= alpha)
     {
-      vtkm::Float32 intensity = 0.4f;
-      vtkm::Vec<vtkm::Float32, 4> blendedColor;
-      vtkm::Vec<vtkm::Float32, 4> srcColor = color;
-      vtkm::Float32 inverseIntensity = (1.0f - intensity);
-      vtkm::Float32 alpha = srcColor[3] * inverseIntensity;
-      blendedColor[0] = 1.0f * intensity + srcColor[0] * alpha;
-      blendedColor[1] = 1.0f * intensity + srcColor[1] * alpha;
-      blendedColor[2] = 1.0f * intensity + srcColor[2] * alpha;
-      blendedColor[3] = 1.0f;
+      constexpr vtkm::Float32 intensity = 0.4f;
+      constexpr vtkm::Float32 inverseIntensity = (1.0f - intensity);
+      alpha *= inverseIntensity;
+      vtkm::Vec<vtkm::Float32, 4> blendedColor(
+        1.0f * intensity + (color[0] * conversionToFloatSpace) * alpha,
+        1.0f * intensity + (color[1] * conversionToFloatSpace) * alpha,
+        1.0f * intensity + (color[2] * conversionToFloatSpace) * alpha,
+        1.0f);
       frameBuffer.Set(offset, blendedColor);
     }
     else
     {
-      frameBuffer.Set(offset, color);
+      // make sure this is opaque
+      vtkm::Vec<vtkm::Float32, 4> fColor((color[0] * conversionToFloatSpace),
+                                         (color[1] * conversionToFloatSpace),
+                                         (color[2] * conversionToFloatSpace),
+                                         1.0f);
+      frameBuffer.Set(offset, fColor);
     }
   }
 
@@ -246,81 +204,6 @@ struct DrawColorBar : public vtkm::worklet::WorkletMapField
   bool Horizontal;
 }; // struct DrawColorBar
 
-struct ColorSwatchExecutor
-{
-  VTKM_CONT
-  ColorSwatchExecutor(vtkm::Id2 dims,
-                      vtkm::Id2 xBounds,
-                      vtkm::Id2 yBounds,
-                      const vtkm::Vec<vtkm::Float32, 4>& color,
-                      const vtkm::cont::ArrayHandle<vtkm::Vec<vtkm::Float32, 4>>& colorBuffer)
-    : Dims(dims)
-    , XBounds(xBounds)
-    , YBounds(yBounds)
-    , Color(color)
-    , ColorBuffer(colorBuffer)
-  {
-  }
-
-  template <typename Device>
-  VTKM_CONT bool operator()(Device) const
-  {
-    VTKM_IS_DEVICE_ADAPTER_TAG(Device);
-
-    vtkm::Id totalPixels = (XBounds[1] - XBounds[0]) * (YBounds[1] - YBounds[0]);
-    vtkm::cont::ArrayHandleCounting<vtkm::Id> iterator(0, 1, totalPixels);
-    vtkm::worklet::DispatcherMapField<DrawColorSwatch, Device> dispatcher(
-      DrawColorSwatch(this->Dims, this->XBounds, this->YBounds, Color));
-    dispatcher.Invoke(iterator, this->ColorBuffer);
-    return true;
-  }
-
-  vtkm::Id2 Dims;
-  vtkm::Id2 XBounds;
-  vtkm::Id2 YBounds;
-  const vtkm::Vec<vtkm::Float32, 4>& Color;
-  const vtkm::cont::ArrayHandle<vtkm::Vec<vtkm::Float32, 4>>& ColorBuffer;
-}; // struct ColorSwatchExecutor
-
-struct ColorBarExecutor
-{
-  VTKM_CONT
-  ColorBarExecutor(vtkm::Id2 dims,
-                   vtkm::Id2 xBounds,
-                   vtkm::Id2 yBounds,
-                   bool horizontal,
-                   vtkm::cont::ArrayHandle<vtkm::Vec<vtkm::Float32, 4>>& colorMap,
-                   const vtkm::cont::ArrayHandle<vtkm::Vec<vtkm::Float32, 4>>& colorBuffer)
-    : Dims(dims)
-    , XBounds(xBounds)
-    , YBounds(yBounds)
-    , Horizontal(horizontal)
-    , ColorMap(colorMap)
-    , ColorBuffer(colorBuffer)
-  {
-  }
-
-  template <typename Device>
-  VTKM_CONT bool operator()(Device) const
-  {
-    VTKM_IS_DEVICE_ADAPTER_TAG(Device);
-
-    vtkm::Id totalPixels = (XBounds[1] - XBounds[0]) * (YBounds[1] - YBounds[0]);
-    vtkm::cont::ArrayHandleCounting<vtkm::Id> iterator(0, 1, totalPixels);
-    vtkm::worklet::DispatcherMapField<DrawColorBar, Device> dispatcher(
-      DrawColorBar(this->Dims, this->XBounds, this->YBounds, this->Horizontal));
-    dispatcher.Invoke(iterator, this->ColorBuffer, this->ColorMap);
-    return true;
-  }
-
-  vtkm::Id2 Dims;
-  vtkm::Id2 XBounds;
-  vtkm::Id2 YBounds;
-  bool Horizontal;
-  vtkm::cont::ArrayHandle<vtkm::Vec<vtkm::Float32, 4>>& ColorMap;
-  const vtkm::cont::ArrayHandle<vtkm::Vec<vtkm::Float32, 4>>& ColorBuffer;
-}; // struct ColorSwatchExecutor
-
 } // namespace internal
 
 struct Canvas::CanvasInternals
@@ -330,11 +213,21 @@ struct Canvas::CanvasInternals
     : Width(width)
     , Height(height)
   {
+    BackgroundColor.Components[0] = 0.f;
+    BackgroundColor.Components[1] = 0.f;
+    BackgroundColor.Components[2] = 0.f;
+    BackgroundColor.Components[3] = 1.f;
+
+    ForegroundColor.Components[0] = 1.f;
+    ForegroundColor.Components[1] = 1.f;
+    ForegroundColor.Components[2] = 1.f;
+    ForegroundColor.Components[3] = 1.f;
   }
 
   vtkm::Id Width;
   vtkm::Id Height;
   vtkm::rendering::Color BackgroundColor;
+  vtkm::rendering::Color ForegroundColor;
   ColorBufferType ColorBuffer;
   DepthBufferType DepthBuffer;
   vtkm::rendering::BitmapFont Font;
@@ -400,6 +293,16 @@ void Canvas::SetBackgroundColor(const vtkm::rendering::Color& color)
   Internals->BackgroundColor = color;
 }
 
+const vtkm::rendering::Color& Canvas::GetForegroundColor() const
+{
+  return Internals->ForegroundColor;
+}
+
+void Canvas::SetForegroundColor(const vtkm::rendering::Color& color)
+{
+  Internals->ForegroundColor = color;
+}
+
 void Canvas::Initialize()
 {
 }
@@ -410,9 +313,9 @@ void Canvas::Activate()
 
 void Canvas::Clear()
 {
-  // TODO: Should the rendering library support policies or some other way to
-  // configure with custom devices?
-  vtkm::cont::TryExecute(internal::ClearBuffersExecutor(GetColorBuffer(), GetDepthBuffer()));
+  internal::ClearBuffers worklet;
+  vtkm::worklet::DispatcherMapField<internal::ClearBuffers> dispatcher(worklet);
+  dispatcher.Invoke(this->GetColorBuffer(), this->GetDepthBuffer());
 }
 
 void Canvas::Finish()
@@ -421,8 +324,9 @@ void Canvas::Finish()
 
 void Canvas::BlendBackground()
 {
-  vtkm::cont::TryExecute(
-    internal::BlendBackgroundExecutor(GetColorBuffer(), GetBackgroundColor().Components));
+  internal::BlendBackground worklet(GetBackgroundColor().Components);
+  vtkm::worklet::DispatcherMapField<internal::BlendBackground> dispatcher(worklet);
+  dispatcher.Invoke(this->GetColorBuffer());
 }
 
 void Canvas::ResizeBuffers(vtkm::Id width, vtkm::Id height)
@@ -460,8 +364,12 @@ void Canvas::AddColorSwatch(const vtkm::Vec<vtkm::Float64, 2>& point0,
   y[1] = static_cast<vtkm::Id>(((point2[1] + 1.) / 2.) * height + .5);
 
   vtkm::Id2 dims(this->GetWidth(), this->GetHeight());
-  vtkm::cont::TryExecute(
-    internal::ColorSwatchExecutor(dims, x, y, color.Components, this->GetColorBuffer()));
+
+  vtkm::Id totalPixels = (x[1] - x[0]) * (y[1] - y[0]);
+  vtkm::cont::ArrayHandleCounting<vtkm::Id> iterator(0, 1, totalPixels);
+  vtkm::worklet::DispatcherMapField<internal::DrawColorSwatch> dispatcher(
+    internal::DrawColorSwatch(dims, x, y, color.Components));
+  dispatcher.Invoke(iterator, this->GetColorBuffer());
 }
 
 void Canvas::AddColorSwatch(const vtkm::Float64 x0,
@@ -502,7 +410,7 @@ void Canvas::AddLine(vtkm::Float64 x0,
 }
 
 void Canvas::AddColorBar(const vtkm::Bounds& bounds,
-                         const vtkm::rendering::ColorTable& colorTable,
+                         const vtkm::cont::ColorTable& colorTable,
                          bool horizontal) const
 {
   vtkm::Float64 width = static_cast<vtkm::Float64>(this->GetWidth());
@@ -516,20 +424,24 @@ void Canvas::AddColorBar(const vtkm::Bounds& bounds,
   vtkm::Id barWidth = x[1] - x[0];
   vtkm::Id barHeight = y[1] - y[0];
 
-  vtkm::cont::ArrayHandle<vtkm::Vec<vtkm::Float32, 4>> colorMap;
   vtkm::Id numSamples = horizontal ? barWidth : barHeight;
+  vtkm::cont::ArrayHandle<vtkm::Vec<vtkm::UInt8, 4>> colorMap;
   colorTable.Sample(static_cast<vtkm::Int32>(numSamples), colorMap);
 
   vtkm::Id2 dims(this->GetWidth(), this->GetHeight());
-  vtkm::cont::TryExecute(
-    internal::ColorBarExecutor(dims, x, y, horizontal, colorMap, this->GetColorBuffer()));
+
+  vtkm::Id totalPixels = (x[1] - x[0]) * (y[1] - y[0]);
+  vtkm::cont::ArrayHandleCounting<vtkm::Id> iterator(0, 1, totalPixels);
+  vtkm::worklet::DispatcherMapField<internal::DrawColorBar> dispatcher(
+    internal::DrawColorBar(dims, x, y, horizontal));
+  dispatcher.Invoke(iterator, this->GetColorBuffer(), colorMap);
 }
 
 void Canvas::AddColorBar(vtkm::Float32 x,
                          vtkm::Float32 y,
                          vtkm::Float32 width,
                          vtkm::Float32 height,
-                         const vtkm::rendering::ColorTable& colorTable,
+                         const vtkm::cont::ColorTable& colorTable,
                          bool horizontal) const
 {
   this->AddColorBar(
@@ -558,7 +470,8 @@ void Canvas::AddText(const vtkm::Matrix<vtkm::Float32, 4, 4>& transform,
                      vtkm::Float32 scale,
                      const vtkm::Vec<vtkm::Float32, 2>& anchor,
                      const vtkm::rendering::Color& color,
-                     const std::string& text) const
+                     const std::string& text,
+                     const vtkm::Float32& depth) const
 {
   if (!Internals->FontTexture.IsValid())
   {
@@ -570,7 +483,7 @@ void Canvas::AddText(const vtkm::Matrix<vtkm::Float32, 4, 4>& transform,
 
   vtkm::rendering::Canvas* self = const_cast<vtkm::rendering::Canvas*>(this);
   TextRenderer fontRenderer(self, Internals->Font, Internals->FontTexture);
-  fontRenderer.RenderText(transform, scale, anchor, color, text);
+  fontRenderer.RenderText(transform, scale, anchor, color, text, depth);
 }
 
 void Canvas::AddText(const vtkm::Vec<vtkm::Float32, 2>& position,
@@ -589,7 +502,7 @@ void Canvas::AddText(const vtkm::Vec<vtkm::Float32, 2>& position,
   vtkm::Matrix<vtkm::Float32, 4, 4> transform =
     vtkm::MatrixMultiply(translationMatrix, vtkm::MatrixMultiply(scaleMatrix, rotationMatrix));
 
-  this->AddText(transform, scale, anchor, color, text);
+  this->AddText(transform, scale, anchor, color, text, 0.f);
 }
 
 void Canvas::AddText(vtkm::Float32 x,
